# Copyright (c) 2012, the Dart project authors.  Please see the AUTHORS file
# for details. All rights reserved. Use of this source code is governed by a
# BSD-style license that can be found in the LICENSE file.

import gfm
from test_case import TestCase


class TestSemiSaneLists(TestCase):
    def setUp(self):
        self.semi_sane_lists = gfm.SemiSaneListExtension()

    def test_doesnt_join_ul_and_ol(self):
        self.assert_renders(
            """
        <ul>
        <li>foo</li>
        <li>bar</li>
        </ul>
        <ol>
        <li>baz</li>
        <li>bip</li>
        </ol>
        """,
            """
        * foo
        * bar

        1. baz
        1. bip
        """,
            [self.semi_sane_lists],
        )

    def test_doesnt_require_blank_line_between_list_types(self):
        self.assert_renders(
            """
        <ol>
        <li>ordered</li>
        <li>also ordered</li>
        </ol>
        """,
            """
        1. ordered
        * also ordered
        """,
            [self.semi_sane_lists],
        )



# Copyright (c) 2012, the Dart project authors.  Please see the AUTHORS file
# for details. All rights reserved. Use of this source code is governed by a
# BSD-style license that can be found in the LICENSE file.

from test_case import TestCase


class TestGfm(TestCase):
    def test_indented_code(self):
        self.assert_renders(
            """
        <p>foo</p>
        <pre><code>some code
        </code></pre>
        <p>bar</p>
        """,
            """
        foo
        
            some code
        
        bar
        """,
            ["mdx_gfm"],
        )

    def test_fenced_code(self):
        test_text = """
        ```
        some code
        ```
        """
        extensions = ["mdx_gfm"]
        if self.has_pygments:
            self.assert_renders(
                """
        <div class="highlight"><pre><span></span><code>some code
        </code></pre></div>
        """,
                test_text,
                extensions,
            )
        else:
            self.assert_renders(
                """
        <pre class="highlight"><code>some code</code></pre>
        """,
                test_text,
                extensions,
            )

    def test_nl2br(self):
        self.assert_renders(
            """
        <p>foo<br />
        bar</p>
        """,
            """
        foo
        bar
        """,
            ["mdx_gfm"],
        )

    def test_smart_emphasis(self):
        self.assert_renders(
            """
        <p>foo__bar__baz</p>
        """,
            """
        foo__bar__baz
        """,
            ["mdx_gfm"],
        )

    def test_table(self):
        self.assert_renders(
            """
        <table>
        <thead>
        <tr>
        <th>First Header</th>
        <th>Second Header</th>
        </tr>
        </thead>
        <tbody>
        <tr>
        <td>Content Cell</td>
        <td>Content Cell</td>
        </tr>
        <tr>
        <td>Content Cell</td>
        <td>Content Cell</td>
        </tr>
        </tbody>
        </table>
        """,
            """
        First Header  | Second Header
        ------------- | -------------
        Content Cell  | Content Cell
        Content Cell  | Content Cell
        """,
            ["mdx_gfm"],
        )

    def test_code_highlighting(self):
        test_text = """
        ```python
        def
        ```
        """
        extensions = ["mdx_gfm"]

        if self.has_pygments:
            self.assert_renders(
                """
        <div class="highlight"><pre><span></span><code><span class="k">def</span>
        </code></pre></div>
        """,
                test_text,
                extensions,
            )
        else:
            self.assert_renders(
                """
        <pre class="highlight"><code class="language-python">def</code></pre>
        """,
                test_text,
                extensions,
            )

    def test_semi_sane_lists(self):
        self.assert_renders(
            """
        <ul>
        <li>foo</li>
        </ul>
        <ol>
        <li>bar</li>
        </ol>
        """,
            """
        * foo

        1. bar
        """,
            ["mdx_gfm"],
        )

    def test_autolink(self):
        self.assert_renders(
            """
        <p><a href="http://foo.com/bar">http://foo.com/bar</a></p>
        """,
            """
        http://foo.com/bar
        """,
            ["mdx_gfm"],
        )

    def test_automail(self):
        self.assert_renders(
            """
        <p><a href="mailto:foo@bar.com">foo@bar.com</a></p>
        """,
            """
        foo@bar.com
        """,
            ["mdx_gfm"],
        )

    def test_strikethrough(self):
        self.assert_renders(
            """
        <p>This is <del>struck</del>.</p>
        """,
            """
        This is ~~struck~~.
        """,
            ["mdx_gfm"],
        )



# Copyright (c) 2013, the Dart project authors.  Please see the AUTHORS file
# for details. All rights reserved. Use of this source code is governed by a
# BSD-style license that can be found in the LICENSE file.

import gfm
from test_case import TestCase


class TestStrikethrough(TestCase):
    def setUp(self):
        self.strikethrough = gfm.StrikethroughExtension()

    def test_double_tilde_strikes(self):
        self.assert_renders(
            """
        <p>This is <del>struck</del>, and so <del>is this</del>.</p>
        """,
            """
        This is ~~struck~~, and so ~~is this~~.
        """,
            [self.strikethrough],
        )

    def test_single_tilde_doesnt_strike(self):
        self.assert_renders(
            """
        <p>This is ~not struck~, and this tilde <del>doesn't~stop~it</del>.</p>
        """,
            """
        This is ~not struck~, and this tilde ~~doesn't~stop~it~~.
        """,
            [self.strikethrough],
        )

    def test_strikethrough_nests(self):
        self.assert_renders(
            """
        <p><del><strong>outer</strong></del>, <strong><del>inner</del></strong>.</p>
        """,
            """
        ~~**outer**~~, **~~inner~~**.
        """,
            [self.strikethrough],
        )

    def test_extra_tildes_dont_cause_strikethrough(self):
        self.assert_renders(
            """
        <p>~~~foo~~~, ~~~bar~~, ~~~~baz~~~~</p>
        """,
            """
        ~~~foo~~~, ~~~bar~~, ~~~~baz~~~~
        """,
            [],
        )



# Copyright (c) 2012, the Dart project authors.  Please see the AUTHORS file
# for details. All rights reserved. Use of this source code is governed by a
# BSD-style license that can be found in the LICENSE file.

import gfm
from test_case import TestCase


class TestAutolink(TestCase):
    def setUp(self):
        self.autolink = gfm.AutolinkExtension()

    def test_autolinks_obvious_links(self):
        self.assert_renders(
            """
        <p><a href="http://foo.com/bar">http://foo.com/bar</a></p>
        """,
            """
        http://foo.com/bar
        """,
            [self.autolink],
        )

    def test_autolinks_uppercase_links(self):
        self.assert_renders(
            """
        <p><a href="HTTP://FOO.COM/BAR">HTTP://FOO.COM/BAR</a></p>
        """,
            """
        HTTP://FOO.COM/BAR
        """,
            [self.autolink],
        )

    def test_autolinks_www_links(self):
        self.assert_renders(
            """
        <p><a href="http://www.foo.com/bar">www.foo.com/bar</a></p>
        """,
            """
        www.foo.com/bar
        """,
            [self.autolink],
        )

    def test_autolinks_ftp_links(self):
        self.assert_renders(
            """
        <p><a href="ftp://foo.com/bar">ftp://foo.com/bar</a></p>
        """,
            """
        ftp://foo.com/bar
        """,
            [self.autolink],
        )

    def test_doesnt_autolink_non_www_links(self):
        self.assert_renders(
            """
        <p>foo.com/bar</p>
        """,
            """
        foo.com/bar
        """,
            [self.autolink],
        )



# Copyright (c) 2012, the Dart project authors.  Please see the AUTHORS file
# for details. All rights reserved. Use of this source code is governed by a
# BSD-style license that can be found in the LICENSE file.

import gfm
from test_case import TestCase


class TestAutomail(TestCase):
    def setUp(self):
        self.automail = gfm.AutomailExtension()

    def test_automails_obvious_mails(self):
        self.assert_renders(
            """
        <p><a href="mailto:foo@bar.com">foo@bar.com</a></p>
        """,
            """
        foo@bar.com
        """,
            [self.automail],
        )

    def test_automails_mails_with_weird_chars(self):
        self.assert_renders(
            """
        <p><a href="mailto:foo+bar@baz.blip.com">foo+bar@baz.blip.com</a></p>
        """,
            """
        foo+bar@baz.blip.com
        """,
            [self.automail],
        )

    def test_automails_uppercase_mails(self):
        self.assert_renders(
            """
        <p><a href="mailto:FOO+BAR@BAZ.BLIP.COM">FOO+BAR@BAZ.BLIP.COM</a></p>
        """,
            """
        FOO+BAR@BAZ.BLIP.COM
        """,
            [self.automail],
        )



# Copyright (c) 2012, the Dart project authors.  Please see the AUTHORS file
# for details. All rights reserved. Use of this source code is governed by a
# BSD-style license that can be found in the LICENSE file.

import unittest
import re

import markdown


class TestCase(unittest.TestCase):
    def setUp(self):
        self.has_pygments = True
        try:
            import pygments  # noqa
        except ImportError:
            self.has_pygments = False

    def assert_renders(self, expected, source, extensions):
        """Asserts that one markdown string renders as expected.

        Takes the expected result, the markdown source, and the extensions to
        use when rendering.
        """

        expected = self.clean_multiline(expected)
        source = self.clean_multiline(source)
        self.assertEqual(expected, markdown.markdown(source, extensions=extensions))

    def clean_multiline(self, text):
        """Cleans an indented multiline string.

        This removes 8 characters of indentation as well as the leading and
        trailing newlines.
        """

        # Optional flags for sub() were only added in python 2.6
        # so compile is used to get the same effect.
        regexPattern = re.compile(r"^ {8}", re.MULTILINE)
        return re.sub(regexPattern, "", text)[1:-1]



# Copyright (c) 2013, the Dart project authors.  Please see the AUTHORS file
# for details. All rights reserved. Use of this source code is governed by a
# BSD-style license that can be found in the LICENSE file.

import gfm
from test_case import TestCase


class TestTaskList(TestCase):
    def setUp(self):
        def cb_attrs(parent, li):
            return {"data-foo": li.text.split("=")[1]}

        def list_attrs(list, depth):
            return {"data-tag": list.tag, "data-depth": str(depth)}

        self.tasklist = gfm.TaskListExtension()
        self.tasklist_ordered_disabled = gfm.TaskListExtension(ordered=False)
        self.tasklist_full_disabled = gfm.TaskListExtension(
            ordered=False, unordered=False
        )
        self.tasklist_no_marker = gfm.TaskListExtension(checked=[], unchecked=[])
        self.tasklist_patterns = gfm.TaskListExtension(
            checked=["~o", "[o]"], unchecked=("~", "[ ]")
        )
        self.tasklist_max_depth = gfm.TaskListExtension(max_depth=2)
        self.tasklist_item_attrs = gfm.TaskListExtension(item_attrs={"class": "foo"})
        self.tasklist_box_attrs = gfm.TaskListExtension(checkbox_attrs={"name": "foo"})
        self.tasklist_box_attrs_cb = gfm.TaskListExtension(checkbox_attrs=cb_attrs)
        self.tasklist_list_attrs = gfm.TaskListExtension(
            list_attrs={"class": "somelist"}
        )
        self.tasklist_list_attrs_cb = gfm.TaskListExtension(list_attrs=list_attrs)

    def test_tasklist_nolist(self):
        self.assert_renders(
            """
        <p>Text</p>
        <ul>
        <li>first item</li>
        </ul>
        """,
            """
        Text

        - first item
        """,
            [self.tasklist],
        )

    def test_tasklist_full_disabled(self):
        self.assert_renders(
            """
        <ul>
        <li>[ ] first item</li>
        <li>[x] second item</li>
        </ul>
        <p>foo</p>
        <ol>
        <li>[ ] first item</li>
        <li>[x] second item</li>
        </ol>
        """,
            """
        - [ ] first item
        - [x] second item

        foo

        1. [ ] first item
        1. [x] second item
        """,
            [self.tasklist_full_disabled],
        )

    def test_tasklist_empty(self):
        self.assert_renders(
            """
        <ul>
        <li>[ ] first item</li>
        <li>[x] second item</li>
        </ul>
        """,
            """
        - [ ] first item
        - [x] second item
        """,
            [self.tasklist_no_marker],
        )

    def test_tasklist_defaults(self):
        self.assert_renders(
            """
        <ul>
        <li><input checked="checked" disabled="disabled" type="checkbox" /> yes item</li>
        <li>item</li>
        <li><input disabled="disabled" type="checkbox" /> no item</li>
        </ul>
        """,
            """
        - [x] yes item
        - item
        - [ ] no item
        """,
            [self.tasklist],
        )

    def test_tasklist_ordered_list(self):
        self.assert_renders(
            """
        <ol>
        <li><input checked="checked" disabled="disabled" type="checkbox" /> yes item</li>
        <li>item</li>
        <li><input disabled="disabled" type="checkbox" /> no item</li>
        </ol>
        """,
            """
        1. [x] yes item
        1. item
        1. [ ] no item
        """,
            [self.tasklist],
        )

    def test_tasklist_whitespace(self):
        self.assert_renders(
            """
        <p>-[ ]not a list</p>
        <ul>
        <li><input disabled="disabled" type="checkbox" />item</li>
        <li><input disabled="disabled" type="checkbox" /> item</li>
        <li><input disabled="disabled" type="checkbox" />  item</li>
        </ul>
        """,
            """
        -[ ]not a list

        - [ ]item
        -  [ ] item
        -   [ ]  item
        """,
            [self.tasklist],
        )

    def test_tasklist_disabled(self):
        self.assert_renders(
            """
        <ul>
        <li><input checked="checked" disabled="disabled" type="checkbox" /> yes item</li>
        <li><input disabled="disabled" type="checkbox" /> no item</li>
        </ul>
        """,
            """
        - [x] yes item
        - [ ] no item
        """,
            [self.tasklist_ordered_disabled],
        )
        self.assert_renders(
            """
        <ol>
        <li>[x] yes item</li>
        <li>[ ] no item</li>
        </ol>
        """,
            """
        1. [x] yes item
        1. [ ] no item
        """,
            [self.tasklist_ordered_disabled],
        )

    def test_tasklist_nested(self):
        self.assert_renders(
            """
        <ul>
        <li><input disabled="disabled" type="checkbox" /> item<ul>
        <li><input disabled="disabled" type="checkbox" /> nested</li>
        <li><input disabled="disabled" type="checkbox" /> nested<ul>
        <li><input disabled="disabled" type="checkbox" /> super nested</li>
        </ul>
        </li>
        </ul>
        </li>
        </ul>
        """,
            """
        - [ ] item
            - [ ] nested
            - [ ] nested
                - [ ] super nested
        """,
            [self.tasklist],
        )

    def test_tasklist_custom_patterns(self):
        self.assert_renders(
            """
        <ul>
        <li><input checked="checked" disabled="disabled" type="checkbox" /> yes item</li>
        <li><input checked="checked" disabled="disabled" type="checkbox" /> yes item</li>
        <li><input disabled="disabled" type="checkbox" /> no item</li>
        <li><input disabled="disabled" type="checkbox" /> no item</li>
        <li>[x] item</li>
        <li>item</li>
        </ul>
        """,
            """
        - [o] yes item
        - ~o yes item
        - [ ] no item
        - ~ no item
        - [x] item
        - item
        """,
            [self.tasklist_patterns],
        )

    def test_tasklist_max_depth(self):
        self.assert_renders(
            """
        <ul>
        <li><input disabled="disabled" type="checkbox" /> item<ul>
        <li><input disabled="disabled" type="checkbox" /> nested 1<ul>
        <li>[ ] super nested 1</li>
        </ul>
        </li>
        <li><input disabled="disabled" type="checkbox" /> nested 2<ul>
        <li>[ ] super nested 2</li>
        </ul>
        </li>
        </ul>
        </li>
        </ul>
        """,
            """
        - [ ] item
            - [ ] nested 1
                - [ ] super nested 1
            - [ ] nested 2
                - [ ] super nested 2
        """,
            [self.tasklist_max_depth],
        )

    def test_tasklist_item_attrs(self):
        self.assert_renders(
            """
        <ul>
        <li class="foo"><input checked="checked" disabled="disabled" type="checkbox" /> yes item</li>
        <li class="foo"><input disabled="disabled" type="checkbox" /> no item</li>
        </ul>
        """,
            """
        - [x] yes item
        - [ ] no item
        """,
            [self.tasklist_item_attrs],
        )

    def test_tasklist_box_attrs(self):
        self.assert_renders(
            """
        <ul>
        <li><input checked="checked" disabled="disabled" name="foo" type="checkbox" /> yes item</li>
        <li><input disabled="disabled" name="foo" type="checkbox" /> no item</li>
        </ul>
        """,
            """
        - [x] yes item
        - [ ] no item
        """,
            [self.tasklist_box_attrs],
        )

    def test_tasklist_box_attrs_cb(self):
        self.assert_renders(
            """
        <ul>
        <li><input checked="checked" data-foo="42" disabled="disabled" type="checkbox" /> foo =42</li>
        <li><input data-foo="1337" disabled="disabled" type="checkbox" /> foo =1337</li>
        </ul>
        """,
            """
        - [x] foo =42
        - [ ] foo =1337
        """,
            [self.tasklist_box_attrs_cb],
        )

    def test_tasklist_list_attrs(self):
        self.assert_renders(
            """
        <ul class="somelist">
        <li><input checked="checked" disabled="disabled" type="checkbox" /> yes item</li>
        <li><input disabled="disabled" type="checkbox" /> no item</li>
        </ul>
        """,
            """
        - [x] yes item
        - [ ] no item
        """,
            [self.tasklist_list_attrs],
        )

        # Non-tasklists are not modified
        self.assert_renders(
            """
        <ul>
        <li>foo</li>
        <li>bar</li>
        </ul>
        """,
            """
        - foo
        - bar
        """,
            [self.tasklist_list_attrs],
        )

    def test_tasklist_list_attrs_nested(self):
        self.assert_renders(
            """
        <ul data-depth="1" data-tag="ul">
        <li><input checked="checked" disabled="disabled" type="checkbox" /> yes item</li>
        <li><input disabled="disabled" type="checkbox" /> no item<ul data-depth="2" data-tag="ul">
        <li><input disabled="disabled" type="checkbox" /> nest 1</li>
        <li><input disabled="disabled" type="checkbox" /> nest 2</li>
        </ul>
        </li>
        </ul>
        """,
            """
        - [x] yes item
        - [ ] no item
            - [ ] nest 1
            - [ ] nest 2
        """,
            [self.tasklist_list_attrs_cb],
        )

    def test_tasklist_list_attrs_cb(self):
        self.assert_renders(
            """
        <ul data-depth="1" data-tag="ul">
        <li><input checked="checked" disabled="disabled" type="checkbox" /> yes item</li>
        <li><input disabled="disabled" type="checkbox" /> no item</li>
        </ul>
        """,
            """
        - [x] yes item
        - [ ] no item
        """,
            [self.tasklist_list_attrs_cb],
        )
        self.assert_renders(
            """
        <ol data-depth="1" data-tag="ol">
        <li><input checked="checked" disabled="disabled" type="checkbox" /> yes item</li>
        <li><input disabled="disabled" type="checkbox" /> no item</li>
        </ol>
        """,
            """
        1. [x] yes item
        1. [ ] no item
        """,
            [self.tasklist_list_attrs_cb],
        )



from setuptools import setup, find_packages

with open("README.rst") as f:
    long_description = f.read()

setup(
    name="py-gfm",
    version="1.0.0",
    description="An implementation of Github-Flavored Markdown written as an "
    "extension to the Python Markdown library.",
    long_description=long_description,
    long_description_content_type="text/x-rst",
    author="Dart Team, Alexandre Macabies",
    author_email="web+oss@zopieux.com",
    url="https://github.com/zopieux/py-gfm",
    packages=find_packages(),
    include_package_data=True,
    install_requires=["markdown~=3.2"],
    data_files=[("", ["LICENSE"])],
    python_requires=">=3.5",
    classifiers=[
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Topic :: Text Processing :: Markup",
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
)
